%%% %%%%%%%%%%%%%%% % Example 1 of the QQ paper titled as 
%%        "Additive Gaussian Process for Computer Models With Qualitative and Quantitative Factors"

%%    iter: the number of simulations
%%    RMSE: a matrix of RMSE for five methods over iter simulations
%%    NSE: a matrix of NSE for five methods over tier simulations
%%    p: the number of quantitative factors
%%    q: the number of qualitative factors
%%    m: a vector of the number of levels in the q qualitative factors

%%  Copyright, 2016, Xinwei Deng and C. Devon Lin .  All rights reserved

iter = 100;
RMSE = zeros(100,1);

p = 6; 
q = 5;
m = [3 3 3 3 3];
nF = 81;
XR = 1; %number of repetitions in XT design
nFF = prod(m);
npred = nF*30;
m2 = zeros(length(m),1);
for i=1:length(m)
   m2(i) = m(i)*(m(i)-1)/2;
end
%%%% OA = load('OA_27_13.txt');
OA = load('OA_81_40.txt');
OA = OA + 1;
IT(1:p) = rand(1,p)*10; %%%starting values for roughness parameters (individual Kriging for each level combination is not feasible in this example)

for i=1:iter
    
 
   %%%%%%%%%%%%%%% design sites %%%%%%%%%%%%%%
   %%%%%%   0~1 scale inputs ,used as input matrix
    XT(:,1:p) = lhsdesign(nF*XR,p);
    XFF = fullfact(m); 
    XF = zeros(nF,q);
    XF(:,1:q) = OA(:,1:q);
    for pt = 1 : XR 
        XT(pt*nF-(nF-1):pt*nF,(p+1):(p+q)) = XF(1:nF,:);
    end
    X = XT;

  %%%%%% original-scale inputs (for calculating true response values only)
  %%% X: design matrix in the original scale
    X(:,1:p) = X(:,1:p)*200-100;
    for ii = 1:length(XT)
        switch XT(ii,p+1)
            case {1} 
                X(ii,p+1) = -50;
            case {2} 
                X(ii,p+1) =  0;
            case {3} 
                X(ii,p+1) = 50;
        end
        switch XT(ii,p+2)
            case {1} 
                X(ii,p+2) = -50;
            case {2} 
                X(ii,p+2) = 0;
            case {3} 
                X(ii,p+2) = 50;
        end    
        switch XT(ii,p+3)
            case {1} 
                X(ii,p+3) = -50;
            case {2} 
                X(ii,p+3) = 0;
            case {3} 
                X(ii,p+3) = 50;
        end
        switch XT(ii,p+4)
            case {1} 
                X(ii,p+4) = -50;
            case {2} 
                X(ii,p+4) = 0;
            case {3} 
                X(ii,p+4) = 50;
        end
        switch XT(ii,p+5)
            case {1} 
                X(ii,p+5) = -50;
            case {2} 
                X(ii,p+5) = 0;
            case {3} 
                X(ii,p+5) = 50;
        end
    end

  %%%% responses at design sites 
  %%% F: the response vector
    XTT = X;
    F =zeros(nF*XR,1);
    for ii =1:floor((p+q)/2);
      F = F + XTT(:,ii).*XTT(:,p+q+1-ii)/4000;  
    end
    F0 = ones(nF*XR,1);
    for ii =1:floor((p+q)/2);
        F0 = F0.*cos(XTT(:,ii)./sqrt(ii)).*sin(XTT(:,p+q+1-ii)./sqrt(ii));
    end
    F = F + F0;
 
 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %% set initial values optimization method can be sensitive to starting values, especially for
  %% high-dimensional cases. Hence, starting values need to be carefully 
  %% chosen. In this example, starting values of cross-correlations in MC/UC are
  %% calculated based on estimated parameters from EC.
    
    options = optimset('Algorithm','active-set','Display','off');
    IT(1:p) = rand(1,p)*10; % starting values for roughness parameters (individual Kriging for each level combination is not feasible in this example)
    

    %%%%% set LB and UB
    %% modelUC is the fitted model using UC method 
    %% to run this code, one can use some modelUC.sigma2 = var(F)
    IT_ADD = [ ones(1,q)*std(F)/q  repmat(IT(1:p),1,q) IT((p+1):end)];
    LB_ADD = ones(1,q+p*q+sum(m.*(m-1)/2))*0.01;
    UB_ADD = [ones(1,q)*3*modelUC.sigma2 ones(1,p*q)*100  (pi-0.01)*ones(1,sum(m.*(m-1)/2))]; 

    [modelUCADD, perfUCADD] = dacefit_QQ_ADDGP(XT, F, q, @regpoly0, @cov_ADD, IT_ADD, LB_ADD, UB_ADD, 'fmincon', options);
 


   %%%%%%%%%%%%%%% prediction%%%%%%%%%%%%%%
   %% 0~1 scale inputs, used as input matrix for Kriging function
    PX(1:npred,1:p) = lhsdesign(npred,p);
    PX(1:npred,(p+1):(p+q)) = repmat(XFF,npred/nFF,1);

   %%% original-scale inputs ,used for calculating true responses at prediction sites
    PXS(:,1:p) = PX(:,1:p)*200-100;
    for ii = 1:length(PX)
        switch PX(ii,p+1)
            case {1} 
                PXS(ii,p+1) = -50;
            case {2} 
                PXS(ii,p+1) = 0;
            case {3} 
                PXS(ii,p+1) = 50;
        end
        switch PX(ii,p+2)
            case {1} 
                PXS(ii,p+2) = -50;
            case {2} 
                PXS(ii,p+2) = 0;
            case {3} 
                PXS(ii,p+2) = 50;
        end    
        switch PX(ii,p+3)
            case {1} 
                PXS(ii,p+3) = -50;
            case {2} 
                PXS(ii,p+3) = 0;
            case {3} 
                PXS(ii,p+3) = 50;
        end
        switch PX(ii,p+4)
            case {1} 
                PXS(ii,p+4) =-50;
            case {2} 
                PXS(ii,p+4) = 0;
            case {3} 
                PXS(ii,p+4) = 50;
        end
        switch PX(ii,p+5)
            case {1} 
                PXS(ii,p+5) = -50;
            case {2} 
                PXS(ii,p+5) = 0;
            case {3} 
                PXS(ii,p+5) = 50;
        end
    end

   %%%%% predicted responses
    PFUCADD = predictor_QQ_ADDGP(PX, modelUCADD);

    TPF =zeros(npred,1);
    for ii =1:floor((p+q)/2);
      TPF = TPF + PXS(:,ii).*PXS(:,p+q+1-ii)/4000;
    end  
    F0 = ones(npred,1);
    for ii =1:floor((p+q)/2);
        F0 = F0.*cos(PXS(:,ii)./sqrt(ii)).*sin(PXS(:,p+q+1-ii)./sqrt(ii));
    end
    TPF = TPF + F0;
   
  % validation
    RMSE(i) = sqrt((sum((PFUCADD-TPF).^2))/npred);

end

 
  
 

